package main

import (
	"fmt"
	"io/ioutil"
	"os"
//	"time"
	"crypto/tls"
	"crypto/x509"

//	"github.com/Wcctnoam/publisher/ghhx"
	MQTT "github.com/eclipse/paho.mqtt.golang"
	gonode "github.com/jgranstrom/gonodepkg"
	json "github.com/jgranstrom/go-simplejson"
//	"golang.org/x/net/context"
)

var (
	c	MQTT.Client
	addr	= "172.20.0.1:1883"
)

func process(cmd *json.Json) (response *json.Json) {
	text := cmd.Get("text").MustString()
	switch text {
		case "Connect":
			Connect()
		case "Subscribe":
			Subscribe("go-mqtt/sample")
		case "Publish":
			Publish("go-mqtt/sample", "This is the message.")
		case "Unsubscribe":
			Unsubscribe("go-mqtt/sample")
		case "Disconnect":
			Disconnect()
		default:
			fmt.Println("Unrecognized operation: ", text)
			panic(1)
	}
	return cmd
}

func main() {
	gonode.Start(process)
/*
	Connect()

	Subscribe("go-mqtt/sample")

	Unsubscribe("go-mqtt/sample")

	Disconnect()
*/
/*	fmt.Println("--------------DELETE--------------")
	resp, err := ghhx.DeleteTable("test4")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ := ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))

	fmt.Println("--------------CREATE--------------")
	resp, err = ghhx.CreateTable("test4")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))

	fmt.Println("--------------CHECK--------------")
	resp, err = ghhx.CheckTable("test4")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))

	fmt.Println("--------------WRITE--------------")
	resp, err := ghhx.WriteRow("test4", "row3", "cf3:cq2", "valueFirst")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ := ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))

	fmt.Println("--------------WRITE2--------------")
	resp, err = ghhx.WriteRow("test4", "row3", "cf3:cq2", "valueSecond")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))
	fmt.Println("--------------ADD--------------")
	resp, err = ghhx.AddFamily("test4", "cf4")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))
	fmt.Println("--------------WRITE3--------------")
	resp, err = ghhx.WriteRow("test4", "row2", "cf3:cq1", "valueC")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))


	fmt.Println("--------------GET ROW--------------")
	resp, err = ghhx.GetRow("test4", "row1")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))

	fmt.Println("--------------GET COLUMN--------------")
	resp, err = ghhx.GetColumn("test4", "row1", "cf2:cq1")
	if err != nil {
		panic(err)
	}

	fmt.Println("response Status:", resp.Status)
	fmt.Println("response Headers:", resp.Header)
	body, _ = ioutil.ReadAll(resp.Body)
	fmt.Println("response Body:", string(body))
*/
}

func NewTlsConfig() *tls.Config {
	// TLS configuration
	certpool := x509.NewCertPool()
	pemCerts, err := ioutil.ReadFile("/home/user/certs/cacert.pem")
	if err == nil {
		certpool.AppendCertsFromPEM(pemCerts)
	}

	cert, err := tls.LoadX509KeyPair("/home/user/certs/client/cert.pem", "/home/user/certs/client/key.pem")
	if err != nil {
		fmt.Println("Error 1")
		panic(err)
	}

	cert.Leaf, err = x509.ParseCertificate(cert.Certificate[0])
	if err != nil {
		fmt.Println("Error 2")
		panic(err)
	}

	return &tls.Config{
		RootCAs: certpool,

		ClientAuth: tls.NoClientCert,
		ClientCAs: nil,

		InsecureSkipVerify: true,

		Certificates: []tls.Certificate{cert},
	}
}


// Message Handler function
var f MQTT.MessageHandler = func(client MQTT.Client, msg MQTT.Message) {
	fmt.Printf("TOPIC: %s\n", msg.Topic())
	fmt.Printf("MSG: %s\n", msg.Payload())
}

func Connect() {
	tlsconfig := NewTlsConfig()

	url := "tcp://" + addr
	opts := MQTT.NewClientOptions().AddBroker(url)
	opts.SetClientID("go-storage").SetTLSConfig(tlsconfig).SetCleanSession(true)
	opts.SetDefaultPublishHandler(f)

	c = MQTT.NewClient(opts)
	if token := c.Connect(); token.Wait() && token.Error() != nil {
		fmt.Println("Error 3")
		fmt.Println(token.Error())
		panic(token.Error())
	}
	fmt.Println("Connected to the server")
}

func Subscribe(topic string){
	if token := c.Subscribe(topic, 0, nil); token.Wait() && token.Error() != nil {
		fmt.Println(token.Error())
		os.Exit(1)
	}
	fmt.Println("Subscribed to ", topic)
}

func Publish(topic, msg string){
	token := c.Publish(topic, 0, false, msg)
	token.Wait()
	fmt.Printf("Published a message.\n")
/*	for i := 0; i < 5 ; i++ {
		text := fmt.Sprintf("This is message #%d!", i)
		token := c.Publish("go-mqtt/sample", 0, false, text)
		token.Wait()
		fmt.Printf("Published message %d.\n", i)
		time.Sleep(10 * time.Second)
	}
*/
}

func Unsubscribe(topic string){
	if token := c.Unsubscribe(topic); token.Wait() && token.Error() != nil {
		fmt.Println(token.Error())
		os.Exit(1)
	}
	fmt.Println("Unsubscribed from ", topic)
}

func Disconnect(){
	c.Disconnect(250)
	fmt.Println("Disconnected from the server")
}
